/**
 * @author Roman Novacek
 * @company HSF, spol. s r. o.
 */

package cz.hsf.novacek;


import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;


public class SpravceUctuImpl implements SpravceUctu {
   
   private final KurzovniListek kurzovniListek;
   
   private Map<String, Ucet> ucty = new HashMap<String, Ucet>();
   
   private DecimalFormat df = new DecimalFormat("000000");
   

   public SpravceUctuImpl(final KurzovniListek kurzovniListek) {
      this.kurzovniListek = kurzovniListek;
   }
   
   void vlozUcet(Ucet u) {
      ucty.put(u.cisloUctu(), u);
   }
   
   Ucet vratUcet(String cisloUctu) throws NeznamyUcetException {
      
      Ucet ucet = ucty.get(cisloUctu);
      
      if (ucet == null)
         throw new NeznamyUcetException();
      
      return ucet;
   }

   public String zalozUcet(Mena mena, double pocatecniVklad) {
      
      if (pocatecniVklad <= 0.0)
         throw new IllegalArgumentException("pocatecniVklad <= 0.0");

      // vytvor novy ucet
      String cisloUctu = df.format(ucty.size() + 1);
      UcetImpl ucet = new UcetImpl(cisloUctu, mena, pocatecniVklad);
      vlozUcet(ucet);
      
      return cisloUctu;
   }

   public void vklad(String ucet, double castka) 
         throws NeznamyUcetException {

      if (castka <= 0.0)
         throw new IllegalArgumentException("castka <= 0.0");
      
      try {
         vratUcet(ucet).pohyb(castka, "vklad");
      } catch (MaloPenezException e) {}
   }

   public void vyber(String ucet, double castka) 
         throws NeznamyUcetException, MaloPenezException {
      
      if (castka <= 0.0)
         throw new IllegalArgumentException("castka <= 0.0");
      
      vratUcet(ucet).pohyb(-castka, "vyber");
   }

   public void prevod(String zUctu, String naUcet, double castka)
   throws NeznamyUcetException, MaloPenezException {

      if (castka <= 0.0)
         throw new IllegalArgumentException("castka <= 0.0");

      Ucet zdrUcet = vratUcet(zUctu);
      Ucet cilUcet = vratUcet(naUcet);

      Mena zdrMena = zdrUcet.mena();
      Mena cilMena = cilUcet.mena();

      if (zdrMena.equals(cilMena)) {
         // prevod ve stejne mene
         zdrUcet.pohyb(-castka, "prevod na ucet " + naUcet);
         cilUcet.pohyb(castka, "prevod z uctu " + zUctu);

      } else {
         // prevod na jinou menu
         double kurz = kurzovniListek.vratKurz(zdrMena, cilMena);
         zdrUcet.pohyb(-castka, "prevod v jine mene na ucet " + naUcet);
         cilUcet.pohyb(castka*kurz, "prevod z uctu " + zUctu);
      }
   }
}
