/**
 * @author Roman Novacek
 * @company HSF, spol. s r. o.
 */

package cz.hsf.novacek;


import com.agical.rmock.extension.junit.RMockTestCase;


public class SpravceUctuImplMockTest extends RMockTestCase {
   
   private String CZK_UCET1;
   private String CZK_UCET2;
   private String EUR_UCET;
   
   private Ucet UCET_100_CZK;
   private Ucet UCET_200_CZK;
   private Ucet UCET_10_EUR;

   private SpravceUctu spravceUctu;
   private KurzovniListek kurzovniListekMock;
   
   public SpravceUctuImplMockTest(String name) {
      super(name);
   }

   @Override
   protected void setUp() throws Exception {
      
      // vytvoreni potrebnych mock objektu
      kurzovniListekMock = (KurzovniListek) mock(KurzovniListek.class, "KurzovniListek");
      
      // vytvoreni spravce uctu
      spravceUctu = new SpravceUctuImpl(kurzovniListekMock);
      
      // zalozeni testovacich uctu
      CZK_UCET1 = spravceUctu.zalozUcet(Mena.CZK, 100.0);
      CZK_UCET2 = spravceUctu.zalozUcet(Mena.CZK, 200.0);
      EUR_UCET = spravceUctu.zalozUcet(Mena.EUR, 10.0);
      UCET_100_CZK = ((SpravceUctuImpl) spravceUctu).vratUcet(CZK_UCET1);
      UCET_200_CZK = ((SpravceUctuImpl) spravceUctu).vratUcet(CZK_UCET2);
      UCET_10_EUR = ((SpravceUctuImpl) spravceUctu).vratUcet(EUR_UCET);
   }

   public void testPrevod_CZKtoCZK() throws NeznamyUcetException, MaloPenezException {

      // neocekava se zadne volani mock objektu
      
      // prepnuti do rezimu verifikace
      startVerification();

      // registrovani potrebnych uctu
      ((SpravceUctuImpl) spravceUctu).vlozUcet(UCET_100_CZK);
      ((SpravceUctuImpl) spravceUctu).vlozUcet(UCET_200_CZK);
      
      // test prevodu
      spravceUctu.prevod(CZK_UCET1, CZK_UCET2, 50.0);
      
      // overeni zustatku
      assertThat(UCET_100_CZK.zustatek(), is.eq(50.0));
      assertThat(UCET_200_CZK.zustatek(), is.eq(250.0));
   }
   
   public void testPrevod_EURtoCZK() throws NeznamyUcetException, MaloPenezException {
      
      // definice ocekavaneho chovani mock objektu
      kurzovniListekMock.vratKurz(null, null);
      modify().args(is.eq(Mena.EUR), is.eq(Mena.CZK)).returnValue(28.51);
      
      // prepnuti do rezimu verifikace
      startVerification();

      // registrovani potrebnych uctu
      ((SpravceUctuImpl) spravceUctu).vlozUcet(UCET_100_CZK);
      ((SpravceUctuImpl) spravceUctu).vlozUcet(UCET_10_EUR);
      
      // vlastni test prevodu
      spravceUctu.prevod(EUR_UCET, CZK_UCET1, 1.0);
      
      // overeni zustatku
      assertThat(UCET_10_EUR.zustatek(), is.eq(9.0));
      assertThat(UCET_100_CZK.zustatek(), is.eq(128.51));
   }

   /**
    * Vytvori specialni Mock objekt, ktery ma "pod sebou" skutecnou instanci objektu UcetImpl.
    * Jako ocekavane chovani "mock" uctu je definovano zcela libovolne volani getteru, kdy
    * se volani presmeruje na skutecny objekt.
    */
   private Ucet mockUcet(String cislo, Mena mena, double zustatek) {

      Object[] args = {cislo, mena, zustatek};
      Ucet ucet = (Ucet) intercept(UcetImpl.class, args, "UcetImpl_" + cislo);

      ucet.cisloUctu(); modify().multiplicity(expect.from(0)).forward();
      ucet.mena(); modify().multiplicity(expect.from(0)).forward();
      ucet.zustatek(); modify().multiplicity(expect.from(0)).forward();

      return ucet;
   }

   public void testPrevod_MockUcty() throws NeznamyUcetException, MaloPenezException {

      // vytvoreni "mock" uctu
      Ucet ucet10EurMock = mockUcet(EUR_UCET, Mena.EUR, 10.0);
      Ucet ucet100CzkMock = mockUcet(CZK_UCET1, Mena.CZK, 100.0);

      // definice ocekavaneho chovani mock objektu
      beginSection(s.ordered("Prevod na jinou menu"));
      {
         kurzovniListekMock.vratKurz(null, null);
         modify().args(is.eq(Mena.EUR), is.eq(Mena.CZK)).returnValue(28.51);
         ucet10EurMock.pohyb(0.0, "");
         modify().args(is.eq(-1.0), is.startingWith("prevod v jine mene na ucet")); //.forward();
         ucet100CzkMock.pohyb(0.0, "");
         modify().args(is.eq(28.51), is.startingWith("prevod z uctu")).forward();
      }
      endSection();

      // prepnuti do rezimu verifikace
      startVerification();

      // registrovani "mock" uctu u spravce
      ((SpravceUctuImpl) spravceUctu).vlozUcet(ucet100CzkMock);
      ((SpravceUctuImpl) spravceUctu).vlozUcet(ucet10EurMock);

      // vlastni test prevodu
      spravceUctu.prevod(EUR_UCET, CZK_UCET1, 1.0);
      
      // overeni zustatku
      // 1. mock ucet vrati PUVODNI zustatek, protoze pri definici ocekavaneho volani metody "pohyb"
      //    nedoslo k presmerovani na skutecnou instanci objektu - nutno odkomentovat "forward()"
      // 2. mock objekt vraci zustatek dle ocekavani
      assertThat(ucet10EurMock.zustatek(), is.eq(9.0));
      assertThat(ucet100CzkMock.zustatek(), is.eq(128.51));
   }
}
