/**
 * @author Roman Novacek
 * @company HSF, spol. s r. o.
 */

package cz.hsf.novacek;


import junit.framework.TestCase;


public class SpravceUctuImplTest extends TestCase {
   
   private String CZK_UCET;
   private String EUR_UCET;
   
   private Ucet UCET_100_CZK;
   private Ucet UCET_10_EUR;
   
   private SpravceUctu spravceUctu;
   
   @Override
   protected void setUp() throws Exception {
      
      // vytvoreni hard-coded mock objektu
      KurzovniListek kurzovniListekMock = new KurzovniListek() {
         
         public double vratKurz(Mena zdrMena, Mena cilMena) {
            
            if (zdrMena != Mena.EUR || cilMena != Mena.CZK)
               throw new IllegalArgumentException("zdrMena != EUR || cilMena != CZK");
            
            return 28.51;
         }
      };
      
      // vytvoreni spravce uctu
      spravceUctu = new SpravceUctuImpl(kurzovniListekMock);
      
      // zalozeni testovacich uctu
      CZK_UCET = spravceUctu.zalozUcet(Mena.CZK, 100.0);
      EUR_UCET = spravceUctu.zalozUcet(Mena.EUR, 10.0);
      UCET_100_CZK = ((SpravceUctuImpl) spravceUctu).vratUcet(CZK_UCET);
      UCET_10_EUR = ((SpravceUctuImpl) spravceUctu).vratUcet(EUR_UCET);
   }
   
   public void testVyber() throws NeznamyUcetException, MaloPenezException {
      
      assertEquals(100.0, UCET_100_CZK.zustatek());
      spravceUctu.vyber(CZK_UCET, 20.0);
      assertEquals(80.0, UCET_100_CZK.zustatek());
      
      try {
         spravceUctu.vyber(CZK_UCET, 100.0);
         fail("MaloPenezException nebyla vyhozena");
      } catch (MaloPenezException e) {}      

      assertEquals(80.0, UCET_100_CZK.zustatek());
   }
   
   public void testPrevod() throws NeznamyUcetException, MaloPenezException {
      
      spravceUctu.prevod(EUR_UCET, CZK_UCET, 1.0);
    
      assertEquals(9.0, UCET_10_EUR.zustatek());
      assertEquals(128.51, UCET_100_CZK.zustatek());
      
      try {
         spravceUctu.prevod(EUR_UCET, CZK_UCET, 10.0);
         fail("MaloPenezException nebyla vyhozena");
      } catch (MaloPenezException e) {}      
      
      assertEquals(9.0, UCET_10_EUR.zustatek());
      assertEquals(128.51, UCET_100_CZK.zustatek());
   }
}
